package com.codigoparallevar.minicards.parts.samples;

import android.graphics.Color;
import android.graphics.Paint;
import android.util.Log;

import com.codigoparallevar.minicards.ScrolledCanvas;
import com.codigoparallevar.minicards.types.connectors.input.InputConnector;
import com.codigoparallevar.minicards.types.Moveable;
import com.codigoparallevar.minicards.types.connectors.input.SignalInputConnector;
import com.codigoparallevar.minicards.types.connectors.output.OutputConnector;
import com.codigoparallevar.minicards.types.Part;
import com.codigoparallevar.minicards.types.PartGrid;
import com.codigoparallevar.minicards.types.RoundInputConnector;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.Collections;
import java.util.List;
import java.util.UUID;

public class Placeholder implements Part {
    private final String _id;
    private final PartGrid _partGrid;
    private int _left;
    private int _top;
    private int _right;
    private int _bottom;

    public Placeholder(String id, PartGrid partGrid, int left, int top, int right, int bottom) {
        _id = id;
        _partGrid = partGrid;
        _left = left;
        _top = top;
        _right = right;
        _bottom = bottom;
    }

    public Placeholder(PartGrid partGrid, int left, int top, int right, int bottom) {
        this(UUID.randomUUID().toString(), partGrid, left, top, right, bottom);
    }

    @Override
    public int getLeft() {
        return _left;
    }

    @Override
    public int getRight() {
        return _right;
    }

    @Override
    public int getTop() {
        return _top;
    }

    @Override
    public int getBottom() {
        return _bottom;
    }

    @Override
    public void draw(ScrolledCanvas canvas, boolean devMode) {
        Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);
        paint.setColor(Color.WHITE);

        // Top
        canvas.drawLine(_left, _top, _right, _top, paint);
        // Bottom
        canvas.drawLine(_left, _bottom, _right, _bottom, paint);
        // Left
        canvas.drawLine(_left, _top, _left, _bottom, paint);
        // Right
        canvas.drawLine(_right, _top, _right, _bottom, paint);
        // Cross, top-left, bottom-right
        canvas.drawLine(_left, _top, _right, _bottom, paint);
        // Cross, top-right, bottom-left
        canvas.drawLine(_right, _top, _left, _bottom, paint);
    }

    @Override
    public void moveEnd(int x, int y) {
        final int width = _right - _left;
        final int height = _bottom - _top;

        _left = x - width / 2;
        _right = _left + width;

        _top = y - height / 2;
        _bottom = _top + height;
    }

    @Override
    public void drop(int x, int y) {
        moveEnd(x, y);
    }

    @Override
    public void touched() {
        Log.d("Placeholder", "Placeholder touched");
    }

    @Override
    public List<SignalInputConnector> getSignalInputConnectors() {
        return Collections.emptyList();
    }

    @Override
    public List<OutputConnector> getOutputConnectors() {
        return Collections.emptyList();
    }

    @Override
    public JSONObject serialize() throws JSONException {
        JSONObject serialized = new JSONObject();

        serialized.put("id", _id);
        serialized.put("left", _left);
        serialized.put("top", _top);
        serialized.put("right", _right);
        serialized.put("bottom", _bottom);

        return serialized;
    }

    @Override
    public void sendSignal(RoundInputConnector roundInputConnector) {
        // @TODO: REMOVE THE NEED FOR THIS
    }

    @Override
    public String getId() {
        return _id;
    }

    @Override
    public InputConnector getConnectorWithId(String inputConnectorId) {
        return null;
    }

    @Override
    public String getConnectorId(InputConnector inputConnector) {
        return null;
    }

    public static Part deserialize(PartGrid partGrid, JSONObject data) throws JSONException {
        String id = data.getString("id");
        int left = data.getInt("left");
        int top = data.getInt("top");
        int right = data.getInt("right");
        int bottom = data.getInt("bottom");

        return new Placeholder(id, partGrid, left, top, right, bottom);
    }

    @Override
    public boolean containsPoint(int x, int y) {
        return (x >= getLeft()) && (x <= getRight())
                && (y >= getTop()) && (y <= getBottom());
    }

    @Override
    public Moveable getMoveable() {
        return this;
    }

    @Override
    public void unlink() {
        for (InputConnector input : getSignalInputConnectors()) {
            input.unlink();
        }
    }
}
